#!/usr/bin/perl

use v5.10;
binmode STDOUT, ":utf8";
use strict;
use warnings;

use XML::Parser;
use Date::Calc qw(Delta_Days);
use HTML::Entities;
#use Data::Dumper;

my %fields = ();
my @MCepoch = (1899, 12, 31);

my $file = shift;

die "Can't find file \"$file\""
  unless -f $file;
    
# Map XML path to MC fields and the MC data type
# MC field data types:
#   1 = list
#   2 = single
my @path_to_MC_field = (
    # list types
    [ 'Collection::DVD::CountryOfOrigin',			'Country',			1 ],
    [ 'Collection::DVD::Genres::Genre',				'Genre', 			1 ],
    [ 'Collection::DVD::Audio::AudioTrack::AudioContent',	'Language',			1 ],
    [ 'Collection::DVD::Studios::Studio',			'Studios',			1 ],

    # single value
    [ 'Collection::DVD::Discs::Disc::Location',			'Filename',			2 ],
    [ 'Collection::DVD::Format::FormatAspectRatio',		'AspectRatio',			2 ],
    [ 'Collection::DVD::Format::FormatVideoStandard',		'Video Standard',		2 ],
    [ 'Collection::DVD::OriginalTitle',				'Original Title',		2 ],
    [ 'Collection::DVD::Rating',			 	'MPAA Rating',			2 ],
    [ 'Collection::DVD::RatingSystem',				'MPAA Rating Description',	2 ],
    [ 'Collection::DVD::Title',			 		'Name',				2 ],
    [ 'Collection::DVD::Overview',			 	'Description',			2 ],
);

my $parser = new XML::Parser(ErrorContext => 2);

$parser->setHandlers(
    Char =>	\&char_handler,
    Start =>	\&start_handler,
    End =>	\&end_handler,
    Final =>	\&final_handler,
    Default =>	\&default_handler);

sub start_handler {
    my $p = shift;
    my $el = shift;

    my $path = join('::', $p->context, $el);
    if ($path eq 'Collection::DVD') {
	%fields = ();
    }

    sub get_values {
	my $attr_re = shift;
	my $ret = '';

	while (@_) {
	    my ($a, $v) = (shift, shift);
	    $ret = ($ret ? join($1 ? '=' : ' ', $ret, $v) : $v)  if $v and $a =~ /$attr_re/;
	}
	return $ret;
    }

    my $parent = $p->current_element;
    if (defined $parent) {

	if ($path eq 'Collection::DVD::Actors::Actor') {
	    push @{$fields{'Actors'}}, get_values('^(?:First|Middle|Last)Name', @_);
	}

	elsif ($path eq 'Collection::DVD::Credits::Credit') {
	    my $val = get_values('^(?:(?:First|Middle|Last)Name)|(CreditSubtype)$', @_);
	    my $key = ($val =~ s/=(Composer|Director|Screenwriter)$//) ? $1 : 'Credits';
	    push @{$fields{$key}}, $val;
	    push @{$fields{'Artist'}}, $val if $key eq 'Director';	# dual mapped
	    push @{$fields{'Music By'}}, $val if $key eq 'Composer';	# dual mapped
	}
    }
}

sub char_handler {
    my ($p, $data) = @_;
    my $parent = $p->current_element;

    my $path = join('::', $p->context);

    # the expat parser returns entities as single characters
    if ($data eq '&' or $data eq '<' or $data eq '>') { $data = encode_entities($data); }
    else					      { $data = decode_entities($data); }

    my ($mcfield,$type);
    if (my @found = grep { $path eq $_->[0] } @path_to_MC_field) {
	($mcfield,$type) = ($found[0][1], $found[0][2]);

	if ($type == 1) {		# lists
	    push @{$fields{$mcfield}}, $data;
	}
	elsif ($type == 2) {	# single value
	    $fields{$mcfield} .= $data;
	}
	else {
	    die "Unexpected type in path_toMC_field";
	}
    }

    elsif ($path =~ /DVD::Audio::AudioTrack::AudioChannels$/) {	# ignoring language for now
	my ($d1,$d2) = ($data =~ /^(\d+)\.(\d)$/);
	$fields{'Channels'} = $d1 + $d2  if $d1;
    }
    elsif ($path =~ /DVD::ProductionYear$/) {
	$fields{'Date'} = 1 + Delta_Days(@MCepoch, ($data,1,1)) if $data ne '0';
    }
    else {
	#print "Parent: $parent, " if defined($parent);
	#say "Data: $data";
    }
}

sub end_handler {
    my $p = shift;
    my $el = shift;

    my $path = join('::', $p->context, $el);
    if ($path eq 'Collection::DVD') {
	$\ = "\r\n";
	$fields{'Media Type'}		= 'Video';
	$fields{'Media Sub Type'}	= 'Movie';
	# sometimes filenames are double-quoted in DVD Profiler.
	$fields{'Filename'} =~ s/"//g if exists $fields{'Filename'};

	print '<Item>';
	for my $key (sort keys %fields) {
	    print '<Field Name="', $key, '">',
		(ref $fields{$key} eq 'ARRAY' ? join('; ', @{$fields{$key}}) : $fields{$key}),
	    '</Field>';
	}
	print '</Item>';
    }
}

sub final_handler {
    #print Dumper(\%fields);
}

sub default_handler {
}

print '<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>', "\r\n";
print '<MPL Version="2.0" Title="JRSidecar">', "\r\n";

$parser->parsefile($file);

print "</MPL>\r\n";
